/***************************************************************************
 *  A/D-Wandler
 ***************************************************************************/
#include <inttypes.h>
#include <avr/io.h>

#include "portconfig.h"


/***************************************************************************
 * 
 * Initialisierung
 * Wandlerfrequenz, Ref-Spannung, Mux-Kanal, Pin als Input
 * 
 ***************************************************************************/
void adcInitialize(void)
{
	/* externe Referenzspannung
	   AD-Wandlerkanal 2 ( ADC2 ) auswhlen 
	   
	   Ergebnis ist Left-Aligned, 
	   d.h. Ergebnis steht in den 8 hherwertigen Bits (reduziert auf 8-bit Genauigkeit!!!)

	 */
	ADMUX = (0<<REFS1)|(0<<REFS0)|(1<<MUX1)|(0<<MUX0)|(1<<ADLAR); 

	

	/* AD-Wandler einschalten und Prescaler einstellen

	   Wandlertaktfrequenz einstellen
	   Sollte zwischen 50 und 200Khz liegen
	   1Mhz/200khz = 5
	   1Mhz/50kHz = 20
	   Prescaler also 8
	*/
	ADCSR = (1<<ADEN)|(0<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);
	
	//Pin als Output mit Pull-Up
	DDRA &= ~(1<<AKKU_VOLTAGE_PIN);
	PORTA |= (1<<AKKU_VOLTAGE_PIN);
}

/***************************************************************************
 * 
 * Wert lesen
 * Der ADC fhrt vor jedem Lesen intern ein Dummy-Readout aus
 * Von daher gengt ein Auslesen
 * 
 ***************************************************************************/
uint8_t adcReadValue(void)
{

	/* Wandlung starten */
	ADCSR |= (1<<ADSC);

	/* Warten bis die AD-Wandlung abgeschloen ist */
	while ( !(ADCSR & (1<<ADIF)) )
			;

	/* 
	 * AD-Wert auslesen 
	 * Messung mehrer Werte, dann mitteln
	 */

	 return ADCH;
}
